#include "boxsir.h"
#include "drawing.h"
#include "opengl.h"
#include "brick.h"
#include <cstdio>

BoxSir::BoxSir()
{
    this->pose = POSE_NORMAL;
    this->normalPose();
    this->position = pos(0., +.3, 0.);
    this->velocity = vec(0., -.01, 0.);
}

BoxSir::~BoxSir()
{
}

void BoxSir::move()
{
    OpenGL * game = OpenGL::Instance();
    pos oldPosition = this->position;
    
    /* Check Animation */
    switch(this->pose){
        case POSE_NORMAL:
            this->normalPose();
            this->animation[MOUTH_ROTATE] = 180;
            this->animation[HAND_LEFT_EXPAND] = 10;
            this->animation[HAND_RIGHT_EXPAND] = 10;
            break;
        case POSE_JUMPING:{
            this->normalPose();
            double expand = 1.8;
            if(this->poseAnimation > 30){
                this->animation[HAND_LEFT_EXPAND] = 10+60*expand-this->poseAnimation*expand;
                this->animation[HAND_RIGHT_EXPAND] = 10+60*expand-this->poseAnimation*expand;
            }else{
                this->animation[HAND_LEFT_EXPAND] = 10+this->poseAnimation*expand;
                this->animation[HAND_RIGHT_EXPAND] = 10+this->poseAnimation*expand;
            }
            this->poseAnimation = (this->poseAnimation + 1)%60;
            break;
        }
        case POSE_HAPPY:{
            this->normalPose();
            this->animation[HEAD_DOWN] = -25;
            this->animation[FACE_TO] = 20;
            double expand = 1.8;
            if(this->poseAnimation > 30){
                this->animation[HAND_LEFT_EXPAND] = 10+60*expand-this->poseAnimation*expand;
                this->animation[HAND_RIGHT_EXPAND] = 10+60*expand-this->poseAnimation*expand;
            }else{
                this->animation[HAND_LEFT_EXPAND] = 10+this->poseAnimation*expand;
                this->animation[HAND_RIGHT_EXPAND] = 10+this->poseAnimation*expand;
            }
            this->poseAnimation = (this->poseAnimation + 1)%60;
            break;
        }
        case POSE_WALK_LEFT:
        case POSE_WALK_RIGHT:
        {
            this->normalPose();
            double expand = 1.8;
            this->animation[FACE_TO] = this->pose==POSE_WALK_LEFT?-90 : 90;
            if(this->poseAnimation > 30){
                this->animation[HAND_LEFT_RAISE] = -15+(60-this->poseAnimation)*expand;
                this->animation[HAND_RIGHT_RAISE] = 25-(60-this->poseAnimation)*expand;
                this->animation[FOOT_RIGHT_RAISE] = -15+(60-this->poseAnimation)*expand;
                this->animation[FOOT_LEFT_RAISE] = 25-(60-this->poseAnimation)*expand;
            }else{
                this->animation[HAND_LEFT_RAISE] = -15+this->poseAnimation*expand;
                this->animation[HAND_RIGHT_RAISE] = 25-this->poseAnimation*expand;
                this->animation[FOOT_RIGHT_RAISE] = -15+this->poseAnimation*expand;
                this->animation[FOOT_LEFT_RAISE] = 25-this->poseAnimation*expand;
            }
            this->poseAnimation = (this->poseAnimation + 1)%60;
            break;
        }
        case POSE_COOL:
            this->normalPose();
            this->animation[FACE_TO] = 45;
            this->animation[MOUTH_ROTATE] = 180;
            this->animation[HEAD_DOWN] = -25;
            this->animation[HAND_LEFT_RAISE] = 10;
            this->animation[HAND_RIGHT_RAISE] = -15;
            this->animation[FOOT_RIGHT_RAISE] = 20;
            break;
        case POSE_SAD:
            this->normalPose();
            this->animation[FACE_TO] = 145;
            this->animation[MOUTH_ROTATE] = 0;
            this->animation[HEAD_DOWN] = +35;
            break;
    }
    
    this->velocity[1] = -.012;
    Object::move();
    
    if(this == game->getPlayer()){
        Brick* o = dynamic_cast<Brick*>(game->checkCollision(this));
        if(o){
            rect r = this->getRect(), t = o->getRect();
            o->mark();
            this->position[1] = t.top + (r.top-r.bottom) + .01;
        }
        
        oldPosition = this->position;
        /* Check Input Key */
        if(game->keyDown(GLUT_KEY_LEFT)){
            if(this->pose != POSE_WALK_LEFT)
                this->setPose(POSE_WALK_LEFT);
            this->position += vec(-.015, .0, .0);
        }else if(game->keyDown(GLUT_KEY_RIGHT)){
            if(this->pose != POSE_WALK_RIGHT)
                this->setPose(POSE_WALK_RIGHT);
            this->position += vec(+.015, .0, .0);
        }else if(game->keyDown(GLUT_KEY_DOWN)){
            this->setPose(POSE_COOL);
        }else if(game->keyDown(GLUT_KEY_UP)){
            this->setPose(POSE_JUMPING);
        }else{
            if(o == NULL)
                this->setPose(POSE_JUMPING);
            else if(this->pose != POSE_NORMAL)
                this->setPose(POSE_NORMAL);
        }
        if(game->checkCollision(this)){
            this->position = oldPosition;
        }
    }
}

void BoxSir::setPose(int index)
{
    if(this->pose != index){
        this->pose = index;
        this->poseAnimation = 0;
    }
}

void BoxSir::normalPose()
{
    for(int i=0; i<sizeof(this->animation)/sizeof(this->animation[0]); i++)
        this->animation[i] = 0.;
}

rect BoxSir::getRect()
{
    rect r;
    r.left = this->position.x() - .1*.3;
    r.right = this->position.x() + .1*.3;
    r.top = this->position.y();
    r.bottom = this->position.y() - .74*.3;
    return r;
}

void BoxSir::draw()
{
    OpenGL* game = OpenGL::Instance();
    glScaled(.3, .3, .3);
    
    Drawing::setMaterial(1, 1, 1);
    Drawing::setMaterial(255./256, 209./256, 222./256);
    glRotated(this->animation[FACE_TO], 0, 1, 0);
    glTranslated(0, -.3, 0);
    
    glPushMatrix();
    glRotated(this->animation[HEAD_DOWN], 1, 0, 0);
    glRotated(this->animation[HEAD_PAN], 0, 1, 0);
    /* Draw head */
    glTranslated(0, .15, 0);
    glScaled(1.2, 1, .9);
    game->bindTexture("front");
    //Drawing::drawBox(.3, 1.);
    GLfloat fsize = .15;
    glBegin(GL_QUADS);
    glNormal3f(0, 0, 1);
    glTexCoord2f(0.0, 1.0); glVertex3f(-fsize, fsize, fsize);
    glTexCoord2f(1.0, 1.0); glVertex3f(fsize, fsize, fsize);
    glTexCoord2f(1.0, 0.0); glVertex3f(fsize, -fsize, fsize);
    glTexCoord2f(0.0, 0.0); glVertex3f(-fsize, -fsize, fsize);
    glEnd();
    game->bindTexture("left");
    glBegin(GL_QUADS);
    glNormal3f(-1, 0, 0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-fsize, fsize, -fsize);
    glTexCoord2f(1.0, 1.0); glVertex3f(-fsize, fsize, fsize);
    glTexCoord2f(1.0, 0.0); glVertex3f(-fsize, -fsize, fsize);
    glTexCoord2f(0.0, 0.0); glVertex3f(-fsize, -fsize, -fsize);
    glEnd();
    // Top
    game->bindTexture("right");
    glBegin(GL_QUADS);
    glNormal3f(0, 1, 0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-fsize, fsize, -fsize);
    glTexCoord2f(1.0, 1.0); glVertex3f(fsize, fsize, -fsize);
    glTexCoord2f(1.0, 0.0); glVertex3f(fsize, fsize, fsize);
    glTexCoord2f(0.0, 0.0); glVertex3f(-fsize, fsize, fsize);
    glEnd();
    // Right
    game->bindTexture("right");
    glBegin(GL_QUADS);
    glNormal3f(1, 0, 0);
    glTexCoord2f(0.0, 1.0); glVertex3f(fsize, fsize, fsize);
    glTexCoord2f(1.0, 1.0); glVertex3f(fsize, fsize, -fsize);
    glTexCoord2f(1.0, 0.0); glVertex3f(fsize, -fsize, -fsize);
    glTexCoord2f(0.0, 0.0); glVertex3f(fsize, -fsize, fsize);
    glEnd();
    // Back
    game->bindTexture("right");
    glBegin(GL_QUADS);
    glNormal3f(0, 0, -1);
    glTexCoord2f(0.0, 1.0); glVertex3f(fsize, fsize, -fsize);
    glTexCoord2f(1.0, 1.0); glVertex3f(-fsize, fsize, -fsize);
    glTexCoord2f(1.0, 0.0); glVertex3f(-fsize, -fsize, -fsize);
    glTexCoord2f(0.0, 0.0); glVertex3f(fsize, -fsize, -fsize);
    glEnd();
    glPopMatrix();
    /* Draw body */
    glPushMatrix();
    glTranslated(0, -.12, .0);
    glScaled(1, 1.2, .5);
    Drawing::drawBox(.2, 1.);
    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
    /* Draw left hand */
    glPushMatrix();
    glRotated(+this->animation[HAND_LEFT_EXPAND], 0, 0, 1);
    glRotated(-this->animation[HAND_LEFT_RAISE], 1, 0, 0);
    glTranslated(+.12, -.15, 0);
    glScaled(.25, 1.5, .5);
    Drawing::drawBox(.2);
    glPopMatrix();
    /* Draw right hand */
    glPushMatrix();
    glRotated(-this->animation[HAND_RIGHT_EXPAND], 0, 0, 1);
    glRotated(-this->animation[HAND_RIGHT_RAISE], 1, 0, 0);
    glTranslated(-.12, -.15, 0);
    glScaled(.25, 1.5, .5);
    Drawing::drawBox(.2);
    glPopMatrix();
    /* Draw left foot */
    glTranslated(.0, -.24, .0);
    glPushMatrix();
    glRotated(-this->animation[FOOT_LEFT_RAISE], 1, 0, 0);
    glTranslated(+.05, -.1, .0);
    glScaled(.8, 2, 1);
    Drawing::drawBox(.1);
    glPopMatrix();
    /* Draw right foot */
    glPushMatrix();
    glRotated(-this->animation[FOOT_RIGHT_RAISE], 1, 0, 0);
    glTranslated(-.05, -.1, .0);
    glScaled(.8, 2, 1);
    Drawing::drawBox(.1);
    glPopMatrix();
    
}

