
#ifdef __WIN32__
#include <windows.h>
#endif
#include "drawing.h"

Drawing::Drawing()
{
}

Drawing::~Drawing()
{
}

void Drawing::drawBox(GLdouble size)
{
    static GLfloat n[6][3] = {
        {-1.0, 0.0, 0.0},
        {0.0, 1.0, 0.0},
        {1.0, 0.0, 0.0},
        {0.0, -1.0, 0.0},
        {0.0, 0.0, 1.0},
        {0.0, 0.0, -1.0}
    };
    static GLint faces[6][4] = {
        {0, 1, 2, 3},
        {3, 2, 6, 7},
        {7, 6, 5, 4},
        {4, 5, 1, 0},
        {5, 6, 2, 1},
        {7, 4, 0, 3}
    };
    GLfloat v[8][3];
    GLint i;
    v[0][0] = v[1][0] = v[2][0] = v[3][0] = -size / 2;
    v[4][0] = v[5][0] = v[6][0] = v[7][0] = size / 2;
    v[0][1] = v[1][1] = v[4][1] = v[5][1] = -size / 2;
    v[2][1] = v[3][1] = v[6][1] = v[7][1] = size / 2;
    v[0][2] = v[3][2] = v[4][2] = v[7][2] = -size / 2;
    v[1][2] = v[2][2] = v[5][2] = v[6][2] = size / 2;
    for (i = 5; i >= 0; i--)
    {
        glBegin(GL_QUADS);
        glNormal3fv(&n[i][0]);
        glVertex3fv(&v[faces[i][0]][0]);
        glVertex3fv(&v[faces[i][1]][0]);
        glVertex3fv(&v[faces[i][2]][0]);
        glVertex3fv(&v[faces[i][3]][0]);
        glEnd();
    }
}

void Drawing::setMaterial(GLfloat r, GLfloat g, GLfloat b)
{
    GLfloat ambientcolor[] = { r/5, g/5, b/5, 1.0 };
    GLfloat diffusecolor[] = { r, g, b, 1.0 };
    GLfloat specularcolor[] = { r*1.5, r*1.5, r*1.5, 1.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambientcolor);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffusecolor);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularcolor);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 64.);
}

void Drawing::drawText(const string str, GLdouble x, GLdouble y, int size)
{
    glRasterPos2d(x, y);
    for (string::const_iterator ii = str.begin();
        ii != str.end();
        ++ii){
        if(size == 10)
            glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, *ii);
        else
            glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *ii);
    }
}

void Drawing::drawBox(GLdouble size, GLdouble textureRepeat)
{
    static GLfloat n[6][3] = {
        {-1.0, 0.0, 0.0},
        {0.0, 1.0, 0.0},
        {1.0, 0.0, 0.0},
        {0.0, -1.0, 0.0},
        {0.0, 0.0, 1.0},
        {0.0, 0.0, -1.0}
    };
    static GLint faces[6][4] = {
        {0, 1, 2, 3},
        {3, 2, 6, 7},
        {7, 6, 5, 4},
        {4, 5, 1, 0},
        {5, 6, 2, 1},
        {7, 4, 0, 3}
    };
    GLfloat v[8][3];
    GLint i;
    v[0][0] = v[1][0] = v[2][0] = v[3][0] = -size / 2;
    v[4][0] = v[5][0] = v[6][0] = v[7][0] = size / 2;
    v[0][1] = v[1][1] = v[4][1] = v[5][1] = -size / 2;
    v[2][1] = v[3][1] = v[6][1] = v[7][1] = size / 2;
    v[0][2] = v[3][2] = v[4][2] = v[7][2] = -size / 2;
    v[1][2] = v[2][2] = v[5][2] = v[6][2] = size / 2;
    GLfloat repeat = textureRepeat;
    for (i = 5; i >= 0; i--)
    {
        glBegin(GL_QUADS);
        glNormal3fv(&n[i][0]);
        glTexCoord2f(0., 0.);
        glVertex3fv(&v[faces[i][0]][0]);
        glTexCoord2f(0., repeat);
        glVertex3fv(&v[faces[i][1]][0]);
        glTexCoord2f(repeat, repeat);
        glVertex3fv(&v[faces[i][2]][0]);
        glTexCoord2f(repeat, 0.);
        glVertex3fv(&v[faces[i][3]][0]);
        glEnd();
    }
}
