#include "musicmixer.h"
#include "SDL/SDL_mixer.h"
#include <iostream>
using namespace std;

MusicMixer* MusicMixer::mm_Instance;

MusicMixer::MusicMixer()
{
	this->bgMusic = 0;
}

MusicMixer::~MusicMixer()
{
	Mix_CloseAudio();
}

void MusicMixer::init()
{
	/**/
	if(Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 1024)==-1) {
		cerr << "Mix_OpenAudio: " << Mix_GetError() << endl;
		exit(2);
	}
}

bool MusicMixer::isMusicStopped()
{
	bool ret = Mix_PausedMusic() || !Mix_PlayingMusic();
	return ret;
}

void MusicMixer::playMusic(string file, bool loop)
{
	stopMusic();
	Mix_FreeMusic(bgMusic);
	bgMusic = Mix_LoadMUS(file.c_str());
	if(bgMusic == NULL){
		cerr << "failed to load music file " << file << " " << Mix_GetError() << endl;
		exit(1);
	}
	if(loop)
		Mix_PlayMusic(bgMusic, -1);
	else
		Mix_PlayMusic(bgMusic, 1);
}

void MusicMixer::stopMusic()
{
	Mix_PauseMusic();
}

void MusicMixer::playSample(string file)
{
    Mix_Volume(-1, 32);
	map<string, Mix_Chunk*>::iterator it = mapSounds.find(file);
	if(it != mapSounds.end())
		Mix_PlayChannel(-1, it->second, 0);
	Mix_Chunk* m = Mix_LoadWAV(file.c_str());
	if(m == NULL){
		cerr << "failed to load wav " << file << " " << Mix_GetError() << endl;
        return;
	}
	mapSounds[file] = m;
	Mix_PlayChannel(-1, m, 0);
}

MusicMixer* MusicMixer::Instance()
{
    if(!mm_Instance)
        mm_Instance = new MusicMixer();
    return mm_Instance;
}

void MusicMixer::Release()
{
    if(mm_Instance)
        delete mm_Instance;
    mm_Instance = 0;
}

