#include "opengl.h"
#include "object.h"
#include "boxsir.h"
#include "brick.h"
#include "drawing.h"
#include "texture.h"
#include "musicmixer.h"
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

OpenGL* OpenGL::ms_instance = 0;

OpenGL::OpenGL()
{
    for(int i=0; i<256; i++)
        this->specialkeys[i] = false;
    this->level = 0;
    this->counter = 0;
    this->mode3d = true;
    this->model = 0;
    this->bgAngle = 0.;
}

OpenGL::~OpenGL()
{
}

OpenGL* OpenGL::Instance()
{
    if(ms_instance == 0) {
        ms_instance = new OpenGL();
    }
    return ms_instance;
}

void OpenGL::Release()
{
    if(ms_instance) {
        delete ms_instance;
    }
    ms_instance = 0;
}

void OpenGL::loop()
{
    glutMainLoop();
}

void OpenGL::onDisplay()
{
    OpenGL* game = OpenGL::Instance();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
    glLoadIdentity();
    glTranslated(0., 0., -3.);
    
    game->drawBackground();
    
    if(game->mode3d){
        glRotated(20, 1, 0, 0);
        glRotated(16, 0, 1, 0);
    }

    for(vector<Object*>::iterator it=game->objects.begin();
        it!=game->objects.end();
        ++it) {
        glPushMatrix();
        const pos &position = (*it)->position;
        glTranslated(position.x(), position.y(), position.z());
        (*it)->draw();
        glPopMatrix();
    }
    game->drawBars();
    glutSwapBuffers();
}

void OpenGL::onIdle()
{
}

void OpenGL::onKeyDown(unsigned char key, int x, int y)
{
    OpenGL* game = OpenGL::Instance();
    switch(key) {
    case 27:
        exit(0);
        break;
    case ' ':
        if(game->stop){
            game->stop = false;
            if(game->lose){
                game->newGame();
            }else{
                if(game->win){
                    game->level ++;
                    game->win = false;
                    for(size_t i=0; i<game->objects.size(); i++){
                        Brick* b = dynamic_cast<Brick*>(game->objects[i]); 
                        if(b){
                            delete game->objects[i];
                            game->objects.erase(game->objects.begin()+i);
                            i--;
                        }
                    }
                }
            }
        }else{
            game->stop = true;
        }
        break;
    }
    glutPostRedisplay();
}

void OpenGL::onMouse(int button, int state, int x, int y)
{
    OpenGL* game = OpenGL::Instance();
    /* Extent selection */
    if(game->mouseX > .3 && game->mouseX < .7 && game->mouseY > -.55 && game->mouseY < -.45){
        if(game->stop)
            OpenGL::onKeyDown(' ', x, y);
    }
}

void OpenGL::onMouseMotion(int x, int y)
{
    OpenGL* game = OpenGL::Instance();
    game->mouseX = (double(x)/game->winw-.5) * (double(game->winw)/double(game->winh)*2);
    game->mouseY = ((1.-double(y)/game->winh)-2.35/4) * 2.25;
    glutPostRedisplay();
}

void OpenGL::onTimer(int value)
{
    OpenGL* game = OpenGL::Instance();
    glutTimerFunc(16, onTimer, value);
    
    game->bgAngle += .003f * (game->level+1);
    if(game->bgAngle >= 360.)
        game->bgAngle = 0.;
    
    
    glutPostRedisplay();

    /* Calculate FPS, why no fps locking in glut? */
    static int time1 = 0, time2, fps = 0;
    time2 = time(0);
    if(time2 - time1 >= 1) {
        game->fps = fps;
        fps = 1, time1 = time2;
    } else
        fps ++;
        
    if(game->stop)
        return;
    /* Object movements */
    for(vector<Object*>::iterator it=game->objects.begin();
        it!=game->objects.end();
        ++it) {
        (*it)->move();
    }
    /* Remove disposed objects */
    for(vector<Object*>::iterator it=game->objects.begin(); it!=game->objects.end();) {
        if((*it)->isDisposed()){
            delete (*it);
            game->objects.erase(it);
        }else
            ++it;
    }
    /* New stages */
    rect r = game->getRect();
    if((game->counter++) % (game->level <= 50 ? (60-game->level) : 20) == 0){
        if(game->currentStage >= MAX_STAGES){
            game->level ++;
            game->generateStages(game->level);
            if(game->level > 0 && game->level % 50 == 0){
                game->stop = true;
                game->win = true;
                MusicMixer::Instance()->playSample("resource/task_complete.wav");
            }
        }else{
            for(int j=0; j<5; j++){
                if(IS_BIT_SET(game->stages[game->currentStage], j)){
                    Brick* b = new Brick();
                    rect br = b->getRect();
                    b->position = pos(r.left + (j+.5)*(br.right-br.left), r.bottom-(r.top-r.bottom)/3, 0.);
                    game->objects.push_back(b);
                }
            }
            ++ game->currentStage;
        }
    }
    /* Check lose */
    rect pr = game->player->getRect();
    if(pr.top < r.bottom-.2 || pr.top > .75){
        game->stop = true;
        game->lose = true;
        MusicMixer::Instance()->playSample("resource/die.wav");
    }
}

void OpenGL::onResize(int w, int h)
{
    OpenGL* game = OpenGL::Instance();
    game->winh = h, game->winw = w;
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // We set up coordinate system so that aspect ratios are always correct,
    //  and the region from -1..1 in x & y always just fits in the viewport.
    glOrtho(-double(w)/h, double(w)/h, -1.25, 1., -20, 20);
    glMatrixMode(GL_MODELVIEW);  // Always go back to modelview mode
    glLoadIdentity();
}

void OpenGL::init(int argc, char* argv [])
{
    /* GLUT windows */
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowPosition(100, 100);
    glutInitWindowSize(500, 500);
    glutCreateWindow(argv[0]);
    glutDisplayFunc(onDisplay);
    if(strstr(argv[0], "2")){
        this->mode3d = false;
    }else{
        this->mode3d = true;
    }
    /* User interacts */
    glutMouseFunc(onMouse);
    glutPassiveMotionFunc(onMouseMotion);
    glutKeyboardFunc(onKeyDown);
    glutReshapeFunc(onResize);
    glutTimerFunc(16, onTimer, 1);
    glutSpecialFunc(onSpecialKey);
    glutSpecialUpFunc(onSpecialKeyUp);
    /* Setup light */
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    this->setLight0();
    
    this->newGame();
    glutPostRedisplay();
    
    /* Load textures */
    glEnable(GL_TEXTURE_2D);
    this->textures["sky"] = new Texture("resource/sky.bmp");
    this->textures["front"] = new Texture("resource/front.bmp");
    this->textures["left"] = new Texture("resource/left.bmp");
    this->textures["right"] = new Texture("resource/right.bmp");
    
    /* Antialiasing */
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glHint (GL_POLYGON_SMOOTH_HINT, GL_NICEST);
    glEnable(GL_POLYGON_SMOOTH);
    //glEnable(GL_MULTISAMPLE_ARB);
}

void OpenGL::setLight0()
{
    GLfloat ambient[] = { 0.1, 0.1, 0.1, 1.0 };
    GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glEnable(GL_LIGHT0);
    GLfloat position[] = { 0, 2.0, 0.0, 3.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, position);
}

void OpenGL::onSpecialKey(int key, int x, int y)
{
    OpenGL* game = OpenGL::Instance();
    if(key < 256)
        game->specialkeys[key] = true;
}

void OpenGL::onSpecialKeyUp(int key, int x, int y)
{
    OpenGL* game = OpenGL::Instance();
    if(key < 256)
        game->specialkeys[key] = false;
}

bool OpenGL::keyDown(int key) const
{
    return this->specialkeys[key];
}

Object* OpenGL::checkCollision(Object* o)
{
    bool hit = false;
    for(vector<Object*>::iterator it=this->objects.begin(); 
        it!=this->objects.end(); it++){
        Brick* brick = dynamic_cast<Brick*>(*it);
        if(o!=brick && brick){
            const rect p = o->getRect(), b = brick->getRect();
            if(p.top > b.bottom && p.bottom < b.top && p.right > b.left && p.left < b.right)
                return *it;
        }
    }
    return 0;
}

rect OpenGL::getRect()
{
    return rect(-1, 1, 1, -1.25);
}

void OpenGL::generateStages(int level)
{
    this->currentStage = 0;
    srand(level);
    memset(this->stages, 0, sizeof(this->stages));
    SET_BIT(this->stages[0], 2);
    for(int i=1; i<MAX_STAGES; i++){
        for(int j=0; j<5; j++){
            if(rand()%2==0 && (j==0||!IS_BIT_SET(this->stages[i], j-1)))
                SET_BIT(this->stages[i], j);
        }
    }
}

int OpenGL::getLevel() const
{
    return this->level;
}

void OpenGL::drawBars()
{
    OpenGL* game = OpenGL::Instance();
    rect r = game->getRect();
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glColor3f(.4, .3, .2);
    glRectf(-2, 2, 2, 1-.2);
    /* Draw edges */
    bool convex = false;
    for(float i=-2; i<2.; i+=.02){
        glBegin(GL_TRIANGLES);
        glVertex3f(i, .8, 0.);
        glVertex3f(i+.01, .8-.05, 0.);
        glVertex3f(i+.02, .8, 0.);
        glEnd();
    }
    char str[100];
    if(game->win){
        glColor3f(1, 0, 0);
        sprintf(str, "Congratulations!!! Current Level:  %02d", this->level);
    }else if(game->lose){
        glColor3f(0, 1, 0);
        sprintf(str, "You Lost!!! Current Level:  %02d", this->level);
    }else{
        glColor3f(.8, .8, .8);
        sprintf(str, "GO Down 50 Levels! Current Level:  %02d", this->level);
    }
    if(game->mode3d){
        Drawing::drawText(str, -.95, .95, 0);
    }else{
        Drawing::drawText(str, -.75, .85, 0);
    }
    char tmp[32];
    glColor3f(.8, .8, .8);
    sprintf(tmp, "FPS: %d", this->fps);
    Drawing::drawText(tmp, -1., 1.1, 10);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    if(this->stop){
        this->drawStart();
    }
}

void OpenGL::drawStart()
{
    /* Show Model */
    if(this->win){
        this->model->setPose(POSE_HAPPY);
    }else if(this->lose){
        this->model->setPose(POSE_SAD);
    }else{
        this->model->setPose(POSE_COOL);
    }
    this->model->move();
    glPushMatrix();
    glTranslated(-.55, .8, 1.);
    glScaled(8, 8, 8);
    this->model->draw();
    glPopMatrix();
    /* Draw Start Button */
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glPushMatrix();
    /* Extent selection */
    if(this->mouseX > .3 && this->mouseX < .7 && this->mouseY > -.55 && this->mouseY < -.45){
        glColor3f(.8, .5, .0);
    }else{
        glColor3f(.4, .1, .1);
    }
    glRectf(.3, -.55, .7, -.45);
    glColor3f(.8, .8, .8);
    Drawing::drawText("Start", .4, -.53, 0);
    Drawing::drawText("This is Xiaoxia's homework for Computer Graphics. Enjoy it!", -1., -1.1, 10);
    Drawing::drawText("Visit my blog at http://xiaoxia.org", -1., -1.15, 10);
    glPopMatrix();
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
}

void OpenGL::newGame()
{
    /* Game initialization */
    this->generateStages(0);
    for(vector<Object*>::iterator it=this->objects.begin(); 
        it!=this->objects.end(); it++){
        delete *it;
    }
    if(this->model)
        delete this->model;
    this->objects.clear();
    this->level = int(this->level/50) * 50;
    this->counter = 0;
    this->model = new BoxSir();
    BoxSir* obj = new BoxSir();
    this->objects.push_back(obj);
    this->player = obj;
    this->stop = true;
    this->win = this->lose = false;
}

void OpenGL::drawBackground()
{
    /* Draw background */
    static GLfloat repeat = 3.;
    glPushMatrix();
    glTranslated(0., 0., -2);
    glRotated(bgAngle, 0, 0, 1.);
    glDisable(GL_LIGHTING);
    this->textures["sky"]->bind();
    glBegin(GL_QUADS);
    glTexCoord2f(.0, .0);
    glVertex3f(-2., 2., 0.);
    glTexCoord2f(repeat, .0);
    glVertex3f(2., 2., 0.);
    glTexCoord2f(repeat, repeat);
    glVertex3f(2., -2., 0.);
    glTexCoord2f(0., repeat);
    glVertex3f(-2., -2., 0.);
    glEnd();
    glEnable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glPopMatrix();
}

void OpenGL::bindTexture(const char* s)
{
    this->textures[s]->bind();
}

BoxSir* OpenGL::getPlayer()
{
    return this->player;
}

