#ifndef OPENGL_H
#define OPENGL_H

#ifdef __WIN32__
#include <windows.h>
#endif
#include <GL/glut.h>
#include "vecpos.h"
using namespace tf;

#include <map>
#include <string>
#include <vector>
using namespace std;
class Object;
class BoxSir;
class Texture;

struct rect{
    double left, right, bottom, top;
    rect(){}
    rect(double l, double t, double r, double b){
        this->left = l, this->right = r, this->bottom = b, this->top = t;
    }
};

#define IS_BIT_SET(a, b) ((a) & (1<<(b)))
#define SET_BIT(a, b) (a|=(1<<(b)))
#define MAX_STAGES 5

class OpenGL {
	static OpenGL* ms_instance;
    BoxSir* player;
    bool specialkeys[256];
    vector<Object*> objects;
    int currentStage;
    unsigned char stages[MAX_STAGES];
    int counter;
    int level;
    bool stop, win, lose;
    bool mode3d;
    double mouseX, mouseY;
    int winw, winh;
    BoxSir* model;
    map<string, Texture*> textures;
    float bgAngle;
    int fps;
public:
	static OpenGL* Instance();
	static void Release();
public:
    void generateStages(int level);
    void init(int argc, char* argv[]);
    void loop();
    void setLight0();
    void newGame();
    void drawBars();
    void drawBackground();
    void drawStart();
    void bindTexture(const char* s);
    bool keyDown(int key) const;
    Object* checkCollision(Object* o);
    BoxSir* getPlayer();
    rect getRect();
    int getLevel()const;
    static void onDisplay();
    static void onResize(int w, int h);
    static void onMouse(int button, int state, int x, int y);
    static void onMouseMotion(int x, int y);
    static void onKeyDown(unsigned char key, int x, int y);
    static void onSpecialKey(int key, int x, int y);
    static void onSpecialKeyUp(int key, int x, int y);
    static void onIdle();
    static void onTimer(int value);
private:
	OpenGL();
	~OpenGL();

};

#endif // OPENGL_H
